#include <p24FJ64GA002.h>
#include "i2c.h"

void i2c_init(unsigned char brg) {

	if (!brg) {
		brg = 0x9e;
	}

	I2C2BRG = brg;	
	I2C2CON = 0x8000;			// Enable I2C 2ch

	return;
}

void i2c_close(void) {
	I2C2CON = 0x0000;			// disable I2C 2ch
}

void i2c_idle(void) {

	while(I2C2CONbits.SEN || I2C2CONbits.PEN || I2C2CONbits.RCEN ||
		I2C2CONbits.ACKEN || I2C2STATbits.TRSTAT );

	return;

}

void i2c_start(void) {

	i2c_idle();					// I2CoX󂭂̂҂
	I2C2STATbits.I2COV = 0;		// Overflow NA
	IFS3bits.MI2C2IF = 0;
	I2C2CONbits.SEN = 1;			// StartRfBVX^[g
	while(I2C2CONbits.SEN);		// I҂

}

void i2c_restart(void) {

	i2c_idle();					// I2CoX󂭂̂҂
	I2C2STATbits.I2COV = 0;		// Overflow NA
	IFS3bits.MI2C2IF = 0;
	I2C2CONbits.RSEN = 1;			// StartRfBVX^[g
	while(I2C2CONbits.RSEN);		// I҂

}

void i2c_stop(void) {

	i2c_idle();
	I2C2CONbits.PEN = 1;
	while(I2C2CONbits.PEN);

}

void i2c_send(unsigned char dev_addr, unsigned int length, unsigned char *buff) {

	unsigned int left;

	dev_addr = dev_addr << 1;
	left = length;
	IFS3bits.MI2C2IF = 0;
	I2C2TRN = dev_addr;
	while(I2C2STATbits.TBF);
	while(I2C2STATbits.ACKSTAT);
	i2c_idle();

	while(left--) {
		IFS3bits.MI2C2IF = 0;
		I2C2TRN = *buff++;
		while(I2C2STATbits.TBF);
	}

	return;
}

void i2c_recv(unsigned char dev_addr, unsigned int length, unsigned char *buff) {

	unsigned int left;
	unsigned char *p;

	p = buff;
	left = length;
	IFS3bits.MI2C2IF = 0;
	I2C2TRN = ( dev_addr << 1 ) | 0x01;
	while(I2C2STATbits.TBF);
	while(I2C2STATbits.ACKSTAT);
	while(!IFS3bits.MI2C2IF);

	I2C2STATbits.I2COV = 0;		// Overflow NA
	while(left--) {

		I2C2CONbits.RCEN = 1;
		IFS3bits.MI2C2IF = 0;
		while(!IFS3bits.MI2C2IF);
		if (left) {
			I2C2CONbits.ACKDT = 0;
		} else {
			I2C2CONbits.ACKDT = 1;
		}
		I2C2CONbits.ACKEN = 1;
		while(I2C2CONbits.ACKEN);
		*p++ = I2C2RCV;
	}

	return;

}
