//
//
#include <p24FJ64GA002.h>
#include <stdio.h>

#include "delay.h"
#include "bc_ami602.h"
#include "C_LCD.h"
#include "i2c.h"

#include "sample1.h"

#define	U1TX		3		// U1TX @\ԍ		
#define U1RX		5		// U1RX sԍ

// PIC 24F ݒ
// NbN 8MHz * 4
_CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & COE_OFF & ICS_PGx1 & FWDTEN_OFF )
_CONFIG2( IESO_ON & FNOSC_FRCPLL & FCKSM_CSDCMD & OSCIOFNC_OFF & IOL1WAY_ON & I2C1SEL_SEC & POSCMOD_NONE )


void debug( unsigned char *mesg, unsigned char l, unsigned char *p) {

	printf("DEBUG: %s:", mesg);
	while(l--) {
		printf(" %02X", *p++);
	}
	printf("\r\n");
	return;
}


void device_init() {

	//
	//  CPU, W[, foCX̏Ȃǂs
	//
	AD1PCFG = 0xFFFF;							// AiO͖͂gp

	// RA0 - 15: Not use...
	TRISA   = 0xFFFF;   //  default
	ODCA    = 0x0000;

	// RB0:  (Reserved)  [PGD1/EMID1]
	// RB1:  (Reserved)  [PGC1/EMIC1]
	// RB2:  (Reserved)  [I2C SDA]
	// RB3:  (Reserved)  [I2C SCL]
	// RB4:  LCD RW      (output)
	// RB5:  (Reserved)  [SIO RX]
	// RB6:  (Reserved)  [SIO TX]
	// RB7:  AMI602 TRG  (output)
	// RB8:  AMI602 BUSY (input)
	// RB9:  AMI602 RST  (output)
	// RB10: LCD E       (output)
	// RB11: LCD RS      (output)
	// RB12: LCD DB4     (output)
	// RB13: LCD DB5     (output)
	// RB14: LCD DB6     (output)
	// RB15: LCD DB7     (output)

	TRISB   = 0x010F;
	ODCB    = 0x0000;

	//
	//	UART1 
	//
	RPINR18bits.U1RXR = U1RX;	// RX 蓖
	RPOR3bits.RP6R = 	U1TX;	// TX 蓖

	U1BRG = 16;				// 115200bpsɐݒ
	U1MODE = 0x8808;
	U1STA  = 0x0400;

	printf("\r\n***************************************************");
	printf("\r\nInit CPU...\r\n");

	//
	//   I2C2W[̏
	//
	i2c_init(0x9e);

	//
	//  AMI602 ̏
	//
	LATB = 0xffff;
	delay_ms(50);				// Ô߂AMI602҂
	ami602_reset();
	printf("DEBUG: AMI602 Reset\r\n");

	//
	// LCDpl̏
	//
	c_lcd_init();
	printf("DEBUG: LCD Panel Reset\r\n");

	return;

}

void parse_data(unsigned char *buffer, unsigned int *result ) {

	unsigned int raw;

	for (raw = 0; raw < 3; raw++) {
		*result++ = (buffer[raw * 3] << 4 ) + ((buffer[raw * 3 + 1] & 0xf0 ) >> 4);
		*result++ = ((buffer[raw * 3 + 1] & 0x0f ) << 8) + (buffer[raw * 3 + 2]);
	}

}

void display_data(unsigned char *buffer) {

	unsigned char disp[32];
	unsigned int  result[6];
	unsigned int  i;

	parse_data(buffer, result);

	for (i = 0; i < 6; i++) {
		result[i] >>= 4;
	}

	sprintf(disp, "MES 1:%02X 2:%02X 3:%02X", *(result + 0), *(result + 1), *(result + 2));
	c_lcd_line( 1, disp );

	sprintf(disp, "    4:%02X 5:%02X 6:%02X", *(result + 3), *(result + 4), *(result + 5));
	c_lcd_line( 2, disp );

}


void ami602_sample() {

	unsigned int  len;
	unsigned char buffer[32];
	unsigned char p[32];
	unsigned int result[10];

	ami602_reset();
	c_lcd_line(3, "(C) 2010 BeatCraft");
	ami602_active();
	printf("DEBUG: AMI602 Active\r\n");

	ami602_READ_CMD( 
		GET_FIRMWARE,
		GET_FIRMWARE_size,
		buffer,
		GET_FIRMWARE_wait1,
		GET_FIRMWARE_wait2,
		GET_FIRMWARE_wait3
	);

	sprintf(p,"AMI602 Ver:%02X%02X", buffer[1], buffer[2] );
	c_lcd_line(0, p );
	debug( "GET VER", GET_FIRMWARE_size, buffer );

	printf("DEBUG: AMI602 Version: %02X%02X  DATE: %02X%02X/%02d/%02d  PATCH: %03d\r\n",
		buffer[1], buffer[2], buffer[3], buffer[4], buffer[5], buffer[6], buffer[7]);

	if (buffer[1] == 0xff && buffer[2] == 0xff ) {
		c_lcd_line(1,"Error or not found");
		return;
	}


//*******************

	buffer[0] = SET_AEN;
	buffer[1] = 1;
	ami602_WRITE_CMD(
		2,
		buffer,
		SET_AEN_wait1,
		SET_AEN_wait2,
		SET_AEN_wait3
	);
	delay_us(30);
	ami602_active();
	delay_us(20);

	while(1) {
		delay_ms(200);

		ami602_active();
		buffer[0] = REQ_MES;
		ami602_WRITE_CMD(
			1,
			buffer,
			REQ_MES_wait1,
			REQ_MES_wait2,
			REQ_MES_wait3
		);

		while(!AMI602_BUSY);
		ami602_active();
		ami602_READ_CMD(
			GET_MES,
			GET_MES_size,
			buffer,
			GET_MES_wait1,
			GET_MES_wait2,
			GET_MES_wait3
		);

		display_data(buffer);
	}

}

int main() {

	device_init();
	ami602_sample();

	while(1);

	return 0;
}

